#ifndef __DATALINK_H__
#define __DATALINK_H__

#include <stdint.h>

#define HEADER_MARKER 0x42

#define BAD_MARKER -1
#define BAD_TYPE -2
#define BAD_LENGTH -3

#define EVOMARKER 0x42
#define HEADER_LEN sizeof(evoframe_header_t)
#define READ_TIMEOUT 0

typedef struct evoframe_header {
    /* Header */
    uint8_t marker;
    uint8_t seq;
    uint8_t pid;
    uint8_t type;   //Datatype
    uint32_t len_bytes;
    uint32_t vector;

} evoframe_header_t;


typedef struct evoframe {
    evoframe_header_t * header;
    uint8_t * data;
} evoframe_t;


int setup_datalink(void);
int unsetup_datalink(void);

int send_frame(evoframe_t * frame, int payload_size);
evoframe_t * recv_frame(void);


int datalink_write(uint8_t * bytes, int len);
int datalink_read(uint8_t * bytes, int len);

evoframe_header_t * create_frame_header(uint8_t marker, uint32_t length, uint8_t pid, uint8_t type);
evoframe_t * create_frame(uint8_t type, uint8_t * data, uint32_t len);
void delete_frame(evoframe_t * frame);

void print_frame_header(evoframe_header_t * header);
void print_frame_payload(uint8_t * data, int len);

#endif
