library ieee;
use ieee.std_logic_1164.all;

use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

package emb is
    -- Physical constants
    constant PINCFG_MEM_LEN : positive := 8; -- need to store 8 words, for 64 * 2 bits = 128 bits = 8 * 16

    constant MC_ADDR_BITS : positive := 12;  -- address lines from the microcontroller, 0 is not used.
    constant MC_DATA_BITS : positive := 16;

    constant PA_IO_LINES : positive := 40;
    constant PB_IO_LINES : positive := 40;

    constant PA_I_LINES : positive := 10;
    constant PB_I_LINES : positive := 10;

    constant IO_LINES : positive := 68; -- Number of pins used for IO.

    constant USR_DATA_BITS : positive := 16;
    constant USR_ADDR_BITS : positive := 12;

    constant MEM_DATA_BITS : positive := 16;
    constant MEM_ADDR_BITS : positive := 12;
    constant MEM_LENGTH : positive := 4096;  --2**12

    constant NOT_BUSY : std_logic_vector := X"0001";
    constant GEN_BUSY : std_logic_vector := X"0004";

    constant BUSY_REG : std_logic_vector := X"009"; -- Command register.
    constant CMD_REG : std_logic_vector := X"00A"; -- Command register.

    constant CMD_NOOP : std_logic_vector := X"0000";
    constant CMD_CONF_PINS : std_logic_vector := X"0001";
    constant CMD_READ_PINS : std_logic_vector := X"0002";

--    subtype mem_addr is unsigned range 0 to MEM_LENGTH;
    constant CONF_WORDS_BASE : std_logic_vector := X"00D";
    constant PINVALS_BASE : std_logic_vector := X"02B";

    subtype bank_sel_type is std_logic_vector(3 downto 0); -- range 0 to 7;

    subtype counter is integer range 0 to 255;


    -- COMMANDS
    constant CMD_PINCONF : std_logic_vector := X"0001";
end package emb;
