library ieee;
use ieee.std_logic_1164.all;

use work.all;

entity pin_bank is
    generic ( npins : positive );

    port (  clk         : in std_logic;
            rst         : in std_logic;
            bank_sel    : in std_logic;
            write       : in std_logic;
            read        : in std_logic;
            configdata  : in std_logic_vector((2 * npins - 1) downto 0);
            pin_io      : inout std_logic_vector((npins - 1) downto 0);
            pin_val     : out std_logic_vector((npins - 1) downto 0) );
end pin_bank;


architecture rtl of pin_bank is
    signal config_reg : std_logic_vector( ((2 * npins) - 1) downto 0);
    signal pinvalue : std_logic_vector(npins - 1 downto 0);
    signal pinvalue_reg : std_logic_vector(npins - 1 downto 0);
begin
    
    pin_row : for pin_index in 0 to npins - 1 generate
    begin
        pin : entity work.pin
            port map (
                clk => clk,
                cfgbits => config_reg((2*pin_index + 1) downto (2 * pin_index)),
                oval    => pinvalue(pin_index),
                pin_out => pin_io(pin_index) );
    end generate pin_row;
   
    --Capture pin configuration data if write enabled.
    write_cfg_word : process(clk) is
    begin
        if (rising_edge(clk) ) then 
            if (rst = '0') then
                config_reg <= (others => '0');
            else
                if (bank_sel = '0') then        
                    if (write = '0') then
                        config_reg <= configdata;
                    elsif (read = '0') then
                        pin_val <= pinvalue_reg;
                    end if;
                end if;
            end if;
        end if;
    end process;

    -- Always capture pin values on flank in case someone wants to read them
    capture_pin_val : process(clk) is
    begin
        if (rising_edge(clk)) then
            pinvalue_reg <= pinvalue;
        end if;
    end process;
end architecture;
 
