library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.all;
use work.emb.all;

entity pin_control is
    port (  clk         : in std_logic;
            rst         : in std_logic;
            write     : in std_logic;
            read        : in std_logic;
            configdata  : in std_logic_vector(15 downto 0);
            bank_addr : bank_sel_type;
            pin_outdata : out std_logic_vector(7 downto 0);
            pins        : inout std_logic_vector(63 downto 0));
end pin_control;


architecture rtl of pin_control is
    type bank_select_t is array (0 to 7) of std_logic;
    signal bank_select : bank_select_t;

    type bank_outputs_t is array (0 to 7) of std_logic_vector(7 downto 0);
    signal bank_outputs : bank_outputs_t;
begin
    
    -- Local memory for storing pin configuration
    configmem : entity memory
        generic map (
            width => MC_DATA_BITS,
            length => PINCFG_MEM_LEN,
            addr_bits => bank_sel_type )
        
        port map (
            clk => clk,
            we => write,
            a => bank_addr,
            d_in => 


                
	


    -- bank select 
    bank_sel : for bank in 0 to 7 generate 
    begin
        --bank_select(bank) <= '0' when (unsigned(bank_addr)) = bank else '1';
        bank_select(bank) <= '0' when to_integer(unsigned(bank_addr)) = bank else '1';
    end generate;

    pin_outdata <= bank_outputs(to_integer(unsigned(bank_addr)));

    pin_banks : for bank_index in 0 to 7 generate -- 8 banks (8 * 8 = 64)
    begin
--        normal_bank : if (bank_index < 7) generate 
            bank : entity pin_bank
                generic map ( npins => 8)
                port map (
                    clk         => clk,
                    rst         => rst,
                    bank_sel    => bank_select(bank_index),
                    read        => read,
                    write       => write,
                    configdata  => configdata,
                    pin_io      =>   pins ((((bank_index + 1) * 8) - 1) downto (bank_index * 8)),
                    pin_val     => bank_outputs(bank_index) );
 --       end generate;

--        last_bank : if (bank_index = 8) generate
--            bank : entity pin_bank 
--                generic map ( npins => 4)
--                port map (
--                    clk         => clk,
--                    rst         => rst,
--                    bank_sel    => bank_select(bank_index),
--                    read        => read,
--                    write       => write,
--                    configdata  => configdata(7 downto 0),
--                    pin_io      => pins(67 downto 64),
--                    pin_val     => bank_outputs(bank_index)(3 downto 0));
--        end generate;
    end generate;
end rtl;


