#include "datalink.h"
#include "virtual_uart.h"
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h> //byte-conversion

int global_sendt = 0;

int unsetup_datalink(void) 
{
    unsetup_virtual_port();
    return 0;
}


int setup_datalink(void)
{
    if(setup_virtual_port() < 0) {
        fprintf(stderr, "setup_datalink: Could not open datalink port\n");
        return -1;
    } 
    return 0;
}

int send_frame(evoframe_t * frame, int payload_size)
{
    //First, send the header.
    uint8_t * head = (uint8_t*)frame->header;
    datalink_write(head, sizeof(evoframe_header_t));
    
//    printf("Sending %u bytes of payload data\n", payload_size);
    datalink_write(frame->data, payload_size);
}

evoframe_t * recv_frame()
{
    evoframe_t * frame = malloc(sizeof(evoframe_t));
    void * head = malloc(sizeof(evoframe_header_t));

    //First, read the header which is fixed-size
    datalink_read(head, sizeof(evoframe_header_t));

    //Cast to a header type.
   evoframe_header_t * h = (evoframe_header_t *)head;

    if(h->marker != HEADER_MARKER) {
        fprintf(stderr, "Got bad marker in header\n");
    }

    int nbytes = ntohl(h->len_bytes);

    //Then, read the number of bytes that the payload indicated.
    frame->header = h;
    frame->data = malloc(nbytes);
    datalink_read(frame->data, nbytes);

    return frame;
}

int datalink_write(uint8_t * bytes, int len) 
{
    int bytes_written = 0;
    
    if (bytes == NULL || len < 1)  {
//        fprintf(stderr, "I don't wanna print nulldata\n");
    }
    
    while(bytes_written < len) {
        if (vport_write_byte(bytes[bytes_written]) < 0) {
            fprintf(stderr, "datalink_write: Could not write byte %d\n", bytes_written);
        } else {
//            printf("Wrote byte %d: %x\n", bytes_written, bytes[bytes_written]);
            bytes_written++;
            global_sendt++;
//            printf("bytes written: %d\n", global_sendt);
        }
    }
}

int datalink_read(uint8_t * bytes, int len)
{
    int bytes_read = 0;
    while(bytes_read < len) {
        bytes[bytes_read] = vport_read_byte();
        bytes_read++;
    }
}

evoframe_header_t * create_frame_header(uint8_t marker, uint32_t length, uint8_t pid, uint8_t type)

{
    
    evoframe_header_t * head = malloc(sizeof(evoframe_header_t));
    head->marker = marker;
    head->len_bytes = htonl(length);
    head->pid = pid;
    head->type = type;

    return head;
}


evoframe_t * create_frame(uint8_t type, uint8_t * data, uint32_t len)
{
    evoframe_t * frame = malloc(sizeof(evoframe_t));
    frame->header = (create_frame_header(EVOMARKER, len, 0, type));
    frame->data = data;
    
    return frame;
}

void print_frame_header(evoframe_header_t * header)
{
    printf("\n--Header--\n");
    printf("Marker: 0x%x\n", header->marker);
    printf("Length (pure bytes): %u\n", header->len_bytes);
    printf("Length (endian converted bytes): %u\n", ntohl(header->len_bytes));
    printf("Sequence number: %u\n", header->seq);
    printf("Pid :%u\n", header->pid);
    printf("Type :%u\n", header->type); 
}


