#ifndef __EVO_EMB_H__
#define __EVO_EMB_H__

#include <stdint.h>
#include <stdio.h>
#include "pin.h"

#define CMD_SEND_PATTERN 0x1
#define CMD_READ_PATTERN 0x2
#define CMD_CONF_FPGA 0x3
#define CMD_RESPONSE 0x4

#define CMD_ALLOC 0x5
#define CMD_STORE_VECTOR 0x6
#define CMD_APPLY_VECTOR 0x7
#define CMD_READ_VECTOR 0x8
#define CMD_DETECT_STABILITY 0x9
#define CMD_APPLY_VECTOR_SET 0xA


#define RESP_OK 0x0   //Sent back if command executed properly.
#define RESP_ALLOC_FAILED 0x1   //Sent back if command executed properly.

/* A pattern is a 2*n-bit string of hexadecimal numbers, where each 2 bits
   correspond to one pin. 
  
   The pins are organized from left to right, pins 0 to 7 corresponds to the first 4 numbers,
   8 to 15 to the next 4, etc. 
*/

/* Set pattern on the Material Bay output pins.
   - len here refers to the number of pins  */
int setPattern(pattern_t * pattern, pinconfig_t * config);
pattern_t * readPattern(pinconfig_t * config);
pattern_t * setWaitRead(pattern_t * apply_pattern, pinconfig_t * config, int wait);

/* Functions for calculating bytes required for storing patterns */
int getNumBytesInput(pinconfig_t * pat);
int getNumBytesOutput(pinconfig_t * pat);

/* Board I/O-functions */
int allocatePatterns(int num, pinconfig_t * pinconfig);
int storePattern(pattern_t * pattern, int n, pinconfig_t * conf);
int storeRndPattern(int num, pinconfig_t * conf);
pattern_t * applyPattern(int num, pinconfig_t * conf, int wait);
pattern_t * loadPattern(int num, pinconfig_t * conf);
pattern_t ** applyAllPatterns(int wait, int n, pinconfig_t * cfg);

/* Stability detection functions */
int detectStabilityPattern(uint32_t samplePeriod, uint32_t stabilityPeriod, 
                       uint32_t timeout, pattern_t * pat, 
                      pinconfig_t * config);

int detectStabilityHighZ(int sampleRate, int stabilityPeriod, int timeout);
/* Obtain a response from the board */
int getResponse();
/* Allocates and generates a random pattern with bytes byte */
pattern_t * generate_random_pattern(int bytes);
/* Frees all patterns in nbs */
void free_patterns(pattern_t ** nbs, int n);

#endif
