#include "fpga_config.h"
#include <stdint.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>

#include "datalink.h"
#include "emb.h"

void config_fpga(const char * path, uint32_t chunksize) 
{
    uint32_t numbytes;
    int chunks, chunk;
    int offset = 0;
    uint8_t * data = read_rbt(path, &numbytes); 

    //Send FPGA_CONFIG command
    send_cmd_fpga_config(numbytes, chunksize);

    //Output data in chunks
    chunks = numbytes / chunksize;
    for(chunk = 0; chunk < chunks; chunk++) {
        send_chunk(data, offset, chunk, chunksize);
        offset += chunksize;
    }
     
    //Send remainder chunk
    send_chunk(data, offset, chunk, numbytes % chunksize);
}


void send_chunk(uint8_t * data, int offset, int chunk, int chunksize)
{
    evoframe_t frame;
    evoframe_header_t header;
    header.marker = 0x42;
    header.len_bytes = htonl(chunksize);
    header.pid = 0x03;
    header.type = 0x3;
    header.seq = htonl(chunk);
    frame.header = &header;

    frame.data = &data[offset];
    printf("sending frame\n"); 
    send_frame(&frame, chunksize);
}


void send_cmd_fpga_config(uint32_t size_of_config, uint32_t chunksize)
{
    uint32_t foo = htonl(size_of_config);
    uint32_t bar = htonl(chunksize);
    
    printf("sending chunksize: %x\n", bar);
    evoframe_t frame;
    evoframe_header_t header;
    header.marker = 0x42;
    header.len_bytes = htonl(8);
    header.pid = 0x42;
    header.type = CMD_CONF_FPGA;
    frame.header = &header;
    frame.data = (uint8_t *)malloc(8); 

    memcpy(frame.data, &foo, 4);
    memcpy(frame.data + 4, &bar, 4);
    
    send_frame(&frame, 8);
}



uint8_t * read_rbt(const char * path, uint32_t * size) {
    char linebuf[256];
    char throwaway[256];
    FILE * file;
    int numbytes;

    if((file = fopen(path, "r")) == NULL) {
        fprintf(stderr, "Could not open file %s\n", path);
        return NULL;
    }
    
    int i;

    //Skip first lines, no fun data.
    for(i = 0; i < 6; i++) {
        fgets(linebuf, 256, file);
    }

    //Parse byte-line
    fgets(linebuf, 256, file);
    sscanf(linebuf, "%s%d", throwaway, &numbytes);
    numbytes /= 8;

    //Malloc room for this array, plus two bytes for 1's.
    numbytes += 2;
    uint8_t * configdata = (uint8_t *)malloc(numbytes);
    *size = numbytes;

    int offset;
    //Next is many lines of 32-bit values. Read line, convert it to
    //an unsigned 32 bit integer, iterate and take each byte into array,
    //in BIG ENDIAN order
    for(offset = 0; offset < numbytes; offset += 4) {
        fgets(linebuf, 256, file); //gets line
        uint32_t n = bin2dec(linebuf);  //note that this assumes \n\r
        uint8_t * n_arr = (uint8_t * )&n;

        int b;
        for(b = 0; b < 4; b++) {
            configdata[offset + b] = n_arr[3-b];
        }
    }
   
    //Append the two last bytes 
    configdata[numbytes - 2] = 0xff; 
    configdata[numbytes - 1] = 0xff; 

    //Debug print
    /*
    int foo = 0;
    int c = 1;
    for(foo = 0; foo < numbytes; foo++) {
        printf("%x, ", configdata[foo]);
        if(c == 4) { c = 0; printf("\n"); }
        c++;
    } */
    return configdata;
}

//Stole this from http://www.daniweb.com/code/snippet216372.html#
uint32_t bin2dec(char *bin)   
{
  uint32_t  b, k, m, n;
  uint32_t  len, sum = 0;
 
  len = strlen(bin) - 2;
  //pruint32_tf("%d\n", len);
  for(k = 0; k <= len; k++) 
  {
    n = (bin[k] - '0'); // char to numeric value
    if ((n > 1) || (n < 0)) 
    {
      puts("\n\n ERROR! BINARY has only 1 and 0!\n");
      return (0);
    }
    for(b = 1, m = len; m > k; m--) 
    {
      // 1 2 4 8 16 32 64 ... place-values, reversed here
      b *= 2;
    }
    // sum it up
    sum = sum + n * b;
    //pruint32_tf("%d*%d + ",n,b);  // uncomment to show the way this works
  }
  return(sum);
}
