#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "config.h"
#include "pin.h"

pinconfig_t * init_config(int n)
{
    pinconfig_t * inited = malloc(sizeof(pinconfig_t));
    inited->npins = n;
    inited->ninput = n;
    inited->noutput = 0;
    inited->pins = malloc(n * sizeof(pin_t));
    int i;
    for(i = 0; i < n; i++) {
        inited->pins[i].mode = PIN_IN;
    }
    return inited;
}

void set_pin_mode(int pin, int mode, pinconfig_t * config) 
{
    int old = config->pins[pin].mode;

    switch(old) {
        case PIN_IN:
            config->ninput--;
            break;
        case PIN_OUT:
            config->noutput--;
            break;
        case PIN_Z:
            config->nz--;
            break;
        default:
            break;
    }

    config->pins[pin].mode = mode;
    switch(mode) {
        case PIN_IN:
            config->ninput++;
            break;
        case PIN_OUT:
            config->noutput++;
            break;
        case PIN_Z:
            config->nz++;
            break;
        default:
            fprintf(stderr, "Attempted to set weird mode\n");
            break;
    }

}

uint8_t * config_to_emb_data(pattern_t * pat, pinconfig_t * conf)
{
    uint8_t * ret = calloc(CFG_VECTOR_BYTES, sizeof(uint8_t));
    uint8_t * pinvals = (uint8_t * )pat->data;
    int i;
    int byte = 0;
    int bit_in_conf_byte = 0;
    int pattern_byte = 0;
    int bit_in_pattern_byte = 0;

    for(i= 0; i < conf->npins; i++) {
        
        //printf("Pattern byte (%d): %X\n", byte, pinvals[byte]);
        
       
        if(conf->pins[i].mode == PIN_IN) {
            //set the bit, because we have a input pin. 
            //printf("Pin %d is input, bit %d in output is 1.\n", i, bit_in_conf_byte);
            SETBIT(ret[byte],bit_in_conf_byte); 
        } else {
            //unset bit, we have an output pin.
            //printf("Pin %d is output, bit %d in output is 0. ", i, bit_in_conf_byte);
            CLRBIT(ret[byte],bit_in_conf_byte);
            
            
            //Now check what values to output on this output pin, and set the corresponding bit.
            if(GETBIT(pinvals[pattern_byte], bit_in_pattern_byte) == 1) {
             //   printf("Consuming bit %d from byte %d and setting it to 1\n", bit_in_pattern_byte, pattern_byte);
                SETBIT(ret[byte], 1 + bit_in_conf_byte); 
            } else {
              //  printf("Consuming bit %d from byte %d and setting it to 0\n", bit_in_pattern_byte, pattern_byte);
                CLRBIT(ret[byte], 1 + bit_in_conf_byte); 
            }

            //We consume an output bit 
            bit_in_pattern_byte += 1;

            if(bit_in_pattern_byte == 8) { 
                pattern_byte++; 
                bit_in_pattern_byte = 0; 
            }


        }



        bit_in_conf_byte += 2; 
        
        if(bit_in_conf_byte == 8) { 
            bit_in_conf_byte = 0; 
            byte++; 
        }
    }

    /*
    printf("So we made %d bytes of configdata\n", byte) ;
    printf("0x");
    for(i = 0; i < 4 * pat->nbytes; i++) {
        printf(" %d:%x", i, ret[i]); 
    }
    printf("\n");
    */ 
    
    return ret;
}


uint8_t * get_input_bytes(uint8_t * data, pinconfig_t * config) 
{
    int i;
    int bit_in_data_byte = 0;   //current bit in input data byte
    int data_byte = 0;          //current byte in input data
    int bit_in_ret_byte = 0;    //current bit in return data byte
    int ret_byte = 0;           //current byte in return data

    //Callocate room for masked return data.
    uint8_t * ret = calloc((int)ceil(config->ninput/(double)8), sizeof(uint8_t));
    //iterate over pins -- if any pins are left in a complete byte, these are left at 0.
    for(i = 0; i < config->npins; i++) {
        if(config->pins[i].mode == PIN_IN) {
            //If bit i is set in the input value, set bit j in return value
            if(GETBIT(data[data_byte], bit_in_data_byte) == 1) {
                SETBIT(ret[ret_byte], bit_in_ret_byte);
            } 
            bit_in_ret_byte++;
            if(bit_in_ret_byte == 8) {
                bit_in_ret_byte = 0;
                ret_byte++;
            }
        }
        
        bit_in_data_byte++;
        if(bit_in_data_byte == 8) {
            bit_in_data_byte = 0;
            data_byte++;
            //data_byte+=2; //we get 16-bits from the fpga.
        }
    }
    return ret;
}

void print_pattern(pattern_t * pattern)
{
    printf("0x");
    int i;
    for(i = 0; i < pattern->nbytes; i++) {
        printf("%02X", pattern->data[i]);
    }
    printf("\n");
}
