#ifndef __PIN_H__
#define __PIN_H__

#include <stdint.h>


#define SETBIT(ADDR, BIT) (ADDR |= (1 << BIT))
#define CLRBIT(ADDR, BIT) (ADDR &= ~(1 << BIT))
#define GETBIT(ADDR, BIT) ((ADDR  & (1 << BIT)) >> BIT)

#define PINS_PER_BYTE 4
#define PIN_OUT 0
#define PIN_IN 1
#define PIN_Z 2

typedef struct pattern {
    int nbytes;
    uint8_t * data;
} pattern_t;

typedef struct pin {
    short mode;
} pin_t;

typedef struct pinconfig {
    int npins;
    int ninput;
    int noutput;
    int nz;
    pin_t * pins;
} pinconfig_t;


pinconfig_t * init_config(int n);

void print_input_string(pattern_t * pattern, pinconfig_t * config);
void set_pin_mode(int pin, int mode, pinconfig_t * config);
/* This function takes a pattern and a configuration, and produces
   a binary blob that configures the pins with the proper IO and data */
uint8_t * config_to_emb_data(pattern_t * pattern,  pinconfig_t * config);
uint8_t * get_input_bytes(uint8_t * data, pinconfig_t * config);

#endif
