#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include <termios.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "virtual_uart.h"

#define BAUDRATE B57600
#define DEVICE "/dev/ttyACM0"

int fd;
struct termios oldtio; 
struct termios settings;

int unsetup_virtual_port()
{
    tcsetattr(fd, TCSANOW, &oldtio);
    return 0;
}


int setup_virtual_port()
{
        fd = open(DEVICE, O_RDWR | O_NOCTTY);
        if ( fd < 0 ) {
                perror(DEVICE);
                return -1;
        }

        tcgetattr(fd, &oldtio);
        bzero(&settings, sizeof(settings));

        settings.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
        settings.c_iflag = IGNPAR;
        settings.c_oflag = 0;
        settings.c_lflag = 0;

        settings.c_cc[VMIN]     = 2;
        settings.c_cc[VTIME]    = 0;

        tcflush(fd, TCIFLUSH);
        tcsetattr(fd, TCSANOW, &settings);
}

int vport_write_byte(uint8_t byte)
{
    int err = 0;
    err |= write(fd, &byte, 1);
    //printf("%X:", byte);
    
    if(err < 0) {
        fprintf(stderr, "Could not write to port: Error %d\n", err);
        fprintf(stderr, "Filedescriptor says %d\n", fd);
        return -1;
    }
    return 0;
}


uint8_t vport_read_byte()
{
    int err = 0;
    uint8_t ret; 
    err = read(fd, &ret, 1);
    
    if(err < 0) {
        fprintf(stderr, "Could not read from port: Error %d\n", err);
        fprintf(stderr, "Filedescriptor says %d\n", fd);
        return -1;
    }
    return (uint8_t)ret;
}
