#include <string.h>
#include "command.h"
#include "datalink.h"
#include "fpga_config.h"
#include "evo_debug.h"
#include "cycle_counter.h"
#include "print_funcs.h"
#include "tc.h"

#include "config.h"
#include "fpga.h"

#include <stdlib.h>
volatile uint32_t time = 0;


int execute_command(evoframe_t * frame )
{
//    print_dbg("Executing command\n");
    switch(frame->header->type) {
        case CMD_FPGA_CONFIG:
            program_fpga(frame);

            break;
        case CMD_SEND_PATTERN:
            //print_dbg("Writing pins\n");
            write_pin_config(frame);
            break; 
        
        case CMD_READ_PINS:
   //         print_dbg("Reading pins\n");
            read_pin_values();
            break;

        case CMD_ALLOC:
  //          print_dbg("Allocing\n");
            alloc_vectors(frame);
            break;

        case CMD_STORE_VECTOR:
 //           print_dbg("Setting vector\n");
            store_vector(frame);
            break;
    
        case CMD_APPLY_VECTOR:
//            print_dbg("Applying vector\n");
            cmd_apply_vector(frame);
            break;

         case CMD_READ_VECTOR:
    //        print_dbg("Reading vector\n");
            read_vector(frame);
            break;

         case CMD_DETECT_STABILITY:
   //         print_dbg("Detecting stability\n");
            detect_stability(frame);
            break;

         case CMD_APPLY_VECTOR_SET:
  //          print_dbg("Applying set of vectors\n");
            cmd_apply_vector_set(frame);
            break;

        default: 
            print_dbg("Not a proper command :< ("); 
            //print_dbg_hex(frame->header->type);
            //print_dbg(")\n");
            break;
    }

    return 0;
}

int write_pin_config(evoframe_t * cmd)
{
    int bank;
    uint16_t * bytes = (uint16_t * )cmd->data;
   
    //Receive the frame containing the config data.
    
    uint16_t * fpga = (uint16_t * )FPGA_ADDR;

    //Configure the banks by writing to the pin-config area of the FPGA
  //  print_dbg("Config writing:\n");
    for(bank = 0; bank < NBANKS; bank++) {
         fpga[PINCFG_OFFSET + bank] = bytes[bank]; 
//         print_dbg_hex(bytes[bank]);
 //        print_dbg("\n");
    }

//    cpu_delay_ms(10, FCPU);
    //Debug, read back data to see if all is well.
   // print_dbg("Config written to banks:\n");
    for(bank = 0; bank < NBANKS; bank++) {
    //     print_dbg_hex(fpga[PINCFG_OFFSET + bank]);
     //    print_dbg("\n");
    }

    fpga[CMD_OFFSET] = CMD_SEND_PATTERN;
    //TODO: Make the FPGA acknowledge that all is fine, return a decent error code if not.
    return 0;
}


int read_pin_values()
{
    uint16_t * fpga = (uint16_t * )FPGA_ADDR;
    int bank; 
    evoframe_t frame;
    evoframe_header_t header;
    header.marker = HEADER_MARKER;
    header.type = CMD_READ_PINS;    
    header.len_bytes = NBANKS * sizeof(uint8_t); 
    frame.header = &header;
    
    //command the FPGA to write the pin values to the shmem
    fpga[CMD_OFFSET] = CMD_READ_PINS;

    frame.data = malloc(NBANKS * sizeof(uint8_t));
    //Ok, data is written back, let's get them to the computar!
    
    uint8_t * tmp_data = (uint8_t*)frame.data;
    //print_dbg("Sending data:\n");
    for(bank = 0; bank < NBANKS; bank++) {
        //drop top byte, 0 anyway.
        tmp_data[bank] = (uint8_t)fpga[PINVAL_OFFSET + bank];
     //   print_dbg_hex(tmp_data[bank]);
      //  print_dbg("\n");
    }
 
    //Respond back that we got the command
    respond(RESP_OK); 
   
    //ship the frame to the host
    send_frame(&frame, NBANKS * sizeof(uint8_t));
    //Deallocate the frame data after use.
    free(frame.data);

    return 0;
}


void respond(int code)
{
    evoframe_t frame;
    evoframe_header_t header;
    header.marker = HEADER_MARKER;
    header.len_bytes = 4;
    header.seq = 0x32;
    header.pid = 0;
    header.type = CMD_RESPONSE;
    frame.header = &header;
    uint8_t * data = (uint8_t*)&code;
    frame.data = data;
    send_frame(&frame, 4);
} 

int alloc_vectors(evoframe_t * frame) 
{
    num_vectors = *((uint32_t*)frame->data);
    int i; 
    configs = malloc(sizeof(uint8_t * ) * num_vectors);
    responses = calloc(num_vectors, sizeof(uint8_t *));
    int failed;

    for(i = 0; i < num_vectors; i++) {
        configs[i] = malloc(CFG_VECTOR_BYTES);
        responses[i] = calloc(sizeof(uint8_t), RESPONSE_BYTES);
        if(configs[i] == NULL || responses[i] == NULL) {
            failed = 1;
         }
    }

    if(failed) {
        respond(RESP_ALLOC_FAILED);
    } else {
        respond(RESP_OK);
    }

    return 0;
}

int store_vector(evoframe_t * frame)
{
    uint32_t n = frame->header->vector; 
//    print_dbg("Setting vector ");
 //   print_dbg_hex(n);
  //  print_dbg("\n");
  
    //Copy the frame data into the config memory.
    //The frame will be freed later on.
    memcpy(configs[n], frame->data, frame->header->len_bytes);

    /*
    print_dbg("\n\nConfig written to vector memory:\n");

    uint16_t bank;
    for(bank = 0; bank < 8; bank++) {
         print_dbg_hex(*((uint16_t *)configs[n] + bank));
         print_dbg("\n");
    }
    */

    respond(RESP_OK);
    return 0;
}



int cmd_apply_vector(evoframe_t * frame)
{
    int n = frame->header->vector;
    uint32_t wait = *((uint32_t*)frame->data);
    
    //The config data is stored on board, just apply this vector.
    apply_vector((uint16_t *)(configs[n]), wait, responses[n]);

    respond(RESP_OK);

    return 0;
}

int read_vector(evoframe_t * frame) 
{
    int n = frame->header->vector;
    uint8_t * bytes = (responses[n]);

    evoframe_t ret;
    evoframe_header_t header;
    header.marker = HEADER_MARKER;
    header.len_bytes = RESPONSE_BYTES ;
    ret.header = &header;
    ret.data = bytes;
   
    send_frame(&ret, RESPONSE_BYTES); 

    return 0;
}


int detect_stability(evoframe_t * frame)
{
    
    uint32_t sample_period; // = 0x200;
    uint32_t timeout; // = 0x1000;
    uint32_t stable; // = 0x500;
   
    uint32_t * daten = (uint32_t*)frame->data;
    memcpy(&sample_period, &(daten[0]), 4);
    memcpy(&stable, &(daten[1]), 4);
    memcpy(&timeout, &(daten[2]), 4);
    print_dbg("magic byte:");
    print_dbg_hex(daten[3]);
    print_dbg("\n");

    print_dbg("detecting stability, period \n");
    print_dbg_hex(sample_period);
    print_dbg(", timeout ");
    print_dbg_hex(timeout);
    print_dbg(", stable ");
    print_dbg_hex(stable);
    print_dbg("\n");

    uint16_t * new_values = malloc(CFG_VECTOR_BYTES);
    uint16_t * last_values = malloc(CFG_VECTOR_BYTES);

        //We idle here until we're stable or we have a timeout.
    //If we've been inside the interrupt routine and
    //fetched new values, we iterate over the fetched
    //values and compare them to what we had last time.

   
    int last_time = 0;
    int delta_time = 0;
    int stable_time = 0;
    int stability_detected = 0;
    int timeout_reached = 0;
    int last_period_start = 0;

    uint16_t * fpga = (uint16_t * )FPGA_ADDR;

    int i;
   
    //Reset time! 
    time = 0;
    
    while(!stability_detected || !timeout_reached) {
        delta_time = (time - last_time);
        last_time = time;
      
        //Read new values. 
        if((time - last_period_start) > sample_period) {
            last_period_start = time;
            for(i = 0; i < NBANKS; i++) {
                fpga[CMD_OFFSET] = CMD_READ_PINS;
                new_values[i] = fpga[PINVAL_OFFSET + i];
            }

            if(equal_values(last_values, new_values, NBANKS)) {
                stable_time += sample_period;
                //print_dbg("stable time:");
                //print_dbg_hex(stable_time);
                //print_dbg("\n");
            } else {
                stable_time = 0;
            }

            //Flip pointers to arrays. 
            uint16_t * tmp = last_values;
            last_values = new_values;
            new_values = tmp;
        }
            
        //Check if we should terminate loop now.
        if(stable_time >= stable) {
            stability_detected = 1;
         //   break;
        }
        
        if(time >= timeout) {
            timeout_reached = 1;
          //  break;
        }
    }
  
    //Stop the clock. 
    //tc_stop(tc, TC_CHANNEL);

    free(last_values);
    free(new_values);

    //If we exited with a stable configuration. 
    if(stability_detected) {
        print_dbg("Stability\n");
        respond(RESP_OK);
        return 0;
    }

    //We timed out, let the world know.
    if(timeout_reached) {
        print_dbg("timeout reached\n");
        respond(RESP_TIMEOUT);
        return 0;
    }

    respond(RESP_OK);
    return 0;
}

int cmd_apply_vector_set(evoframe_t * frame)
{
    int wait = *((uint32_t *)frame->data);
    int i;
    print_dbg("time: ");
    print_dbg_hex(time);
    for(i = 0; i < num_vectors; i++) {
        apply_vector((uint16_t*)configs[i], wait, responses[i]);
    }
    print_dbg("\n time after:");
    print_dbg_hex(time);
    print_dbg("\n");
    return 0;
}

__attribute__((__interrupt__)) void stability_read(void)
{
    time++;
    tc_read_sr(&(AVR32_TC), TC_CHANNEL);
}


int equal_values(uint16_t * a, uint16_t * b, int n)
{
    int i;
    for(i = 0; i < n; i++) {
        if(a[i] != b[i]) {
            return 0;
        }
    }
    print_dbg("equal\n");
    return 1;
}



