#ifndef __COMMAND_H__
#define __COMMAND_H__

#include <stdint.h>
#include "datalink.h"

#define CMD_NOOP 0x0
#define CMD_SEND_PATTERN 0x1
#define CMD_READ_PINS 0x2
#define CMD_FPGA_CONFIG 0x3
#define CMD_RESPONSE 0x4
#define CMD_ALLOC 0x5
#define CMD_STORE_VECTOR 0x6
#define CMD_APPLY_VECTOR 0x7
#define CMD_READ_VECTOR 0x8
#define CMD_DETECT_STABILITY 0x9
#define CMD_APPLY_VECTOR_SET 0xA

#define FPGA_ADDR 0xD0000000
#define CMD_OFFSET 0xA
#define PINCFG_OFFSET 0xD
#define PINVAL_OFFSET 0x02B

#define BUSY_OFFSET 0x9

#define RESP_OK 0x0
#define RESP_ALLOC_FAILED 0x1   //Sent back if command executed properly.
#define RESP_TIMEOUT 0x2

typedef struct command {
    uint8_t command;
    //Data associated with command
    uint32_t size_of_data;
    uint8_t * data;
} command_t;

extern uint32_t num_vectors;
extern uint8_t ** configs;
extern uint8_t ** responses;

//Interface to command managment

/* This attempts to recieve a full command. If timeout is > 0, the function returns
potentially without a new command beeing put in the queue. */
command_t receive_command(int timeout);

/* Attempts to execute a command from the queue. This will free the command after usage, and
also any data associated with the command. */

int execute_command(evoframe_t * cmd);

//Accessors to the queue
int enqueue_command(command_t * cmd);
int dequeue_command(command_t * cmd);

//Util functions for setting various control registers in the FPGA (this is only for my user module though).
void set_wait_time(int wait);

//Commands
int write_pin_config(evoframe_t * cmd);
int read_pin_values();

/* Functions to send status messages back and forth */
void respond(int code);

int alloc_vectors(evoframe_t * frame);
int store_vector(evoframe_t * frame);
int cmd_apply_vector(evoframe_t * frame);
int read_vector(evoframe_t * frame);
int cmd_apply_vector_set(evoframe_t * frame);

int detect_stability(evoframe_t * frame);
__attribute__((__interrupt__)) void stability_read(void);
int equal_values(uint16_t * a, uint16_t * b, int n);

#endif
