#ifndef _CONVERT_H_
#define _CONVERT_H_
#include <stdint.h>

#define TO_DISPLAY(x) to_display[(unsigned char)(x)]

#define REVERSED_FONT

#ifndef REVERSED_FONT

#define DOT (1<<0)
#define A (1<<1)
#define B (1<<2)
#define C (1<<3)
#define D (1<<4)
#define E (1<<5)
#define F (1<<6)
#define G (1<<7)
#define H (1<<14)
#define J (1<<13)
#define K (1<<12)
#define L (1<<11)
#define M (1<<10)
#define N (1<<9)
#define P (1<<8)

#else 

#define DOT 0
#define A (1<<4)
#define B (1<<5)
#define C (1<<6)
#define D (1<<1)
#define E (1<<2)
#define F (1<<3)
#define G (1<<11)
#define H (1<<10)
#define J (1<<9)
#define K (1<<8)
#define L (1<<7)
#define M (1<<14)
#define N (1<<13)
#define P (1<<12)

#endif 

/* 7-bit ascii converter */
uint16_t to_display[128] = {
	/* Control chars. Not really needed */
	0x0, /* NUL */
	0x0, /* SOH */
	0x0, /* STX */
	0x0, /* ETX */
	0x0, /* EOT */
	0x0, /* ENQ */
	0x0, /* ACK */
	0x0, /* BEL */
	0x0, /* BS */
	0x0, /* HT */
	0x0, /* LF */
	0x0, /* VT */
	0x0, /* FF */
	0x0, /* CR */
	0x0, /* SO */
	0x0, /* SI */

	0x0, /* DLE */
	0x0, /* DC1 */
	0x0, /* DC2 */
	0x0, /* DC3 */
	0x0, /* DC4 */
	0x0, /* NAK */
	0x0, /* SYN */
	0x0, /* ETB */
	0x0, /* CAN */
	0x0, /* EM */
	0x0, /* SUB */
	0x0, /* ESC */
	0x0, /* FS */
	0x0, /* GS */
	0x0, /* RS */
	0x0, /* US */

	/* Special characthers */
	0x0,            /* SPACE */
	0x0, /* ! */
	F|G,            /* " */
	0x0, /* # */
	A|C|D|F|G|J|L|N,/* $ */
	0x0, /* % */
	0x0, /* & */
	G,              /* ' */
	0x0, /* ( */
	0x0, /* ) */
	G|H|J|K|L|M|N|P,/* * */
	G|J|L|N,        /* + */
	0x0, /* , */
	J|N,            /* - */
	DOT,            /* . */
	H|M,            /* / */

	/* Numbers */
	A|B|C|D|E|F|H|M,/* 0 */
	B|C,            /* 1 */
	A|B|D|E|J|N,    /* 2 */
	A|B|C|D|J|N,    /* 3 */
	B|C|F|J|N,      /* 4 */
	A|D|F|K|N,      /* 5 */
	A|C|D|E|F|J|N,  /* 6 */
	A|H|M,          /* 7 */
	A|B|C|D|E|F|J|N,/* 8 */
	A|B|C|D|F|J|N,  /* 9 */

	/* Special characthers */
	G|L,            /* : */
	G|M,            /* ; */
	H|K,            /* < */
	D|J|N,          /* = */
	M|P,            /* > */
	A|B|J|L,        /* ? */
	0x0, /* @ */
	
	/* Uppercase letters */
	A|B|C|E|F|J|N,  /* A */
	A|B|C|D|G|J|L,  /* B */
	A|D|E|F,        /* C */
	A|B|C|D|G|L,    /* D */
	A|D|E|F|N,      /* E */
	A|E|F|N,        /* F */
	A|C|D|E|F|J,    /* G */
	B|C|E|F|J|N,    /* H */
	A|D|G|L,        /* I */
	B|C|D|E,        /* J */
	E|F|H|K|N,      /* K */
	D|E|F,          /* L */
	B|C|E|F|H|P,    /* M */
	B|C|E|F|K|P,    /* N */
	A|B|C|D|E|F,    /* O */
	A|B|E|F|J|N,    /* P */
	A|B|C|D|E|F|K,  /* Q */
	A|B|E|F|J|K|N,  /* R */
	A|C|D|F|J|N,    /* S */
	A|G|L,          /* T */
	B|C|D|E|F,      /* U */
	E|F|H|M,        /* V */
	B|C|E|F|K|M,    /* W */
	H|K|M|P,        /* X */
	H|L|P,          /* Y */
	A|D|H|M,        /* Z */

	/* Special characthers */
	A|D|E|F,        /* [ */
	K|P,            /* \ */
	A|B|C|D,        /* ] */
	K|M, /* ^ */
	D,              /* _ */
	P,              /* ` */
	
	/* Lowercase letters */
	/* Some of these looks like uppercase */
	D|E|L|N,        /* a */
	C|D|E|F|J|N,    /* b */
	D|E|J|N,        /* c */
	B|C|D|E|J|N,    /* d */
	A|D|E|F|N,      /* e */
	H|J|L,          /* f */
	A|B|C|D|F|J|N,  /* g */
	C|G|J|L,        /* h */
	L,              /* i */
	B|C|K,          /* j */
	G|H|K|L,        /* k */
	D|E,            /* l */
	C|E|J|L|N,      /* m */
	C|E|J|N,        /* n */
	C|D|E|J|N,      /* o */
	E|F|N|P,        /* p */
	B|C|H|J,        /* q */
	E|N,            /* r */
	D|J|K|N,        /* s */
	J|L|N,          /* t */
	C|D|E,          /* u */
	C|K,            /* v */
	C|E|K|M,        /* w */
	H|K|M|P,        /* x */
	G|L|P,          /* y */
	D|J|M|N,        /* z */
	
	/* More special */
	H|K|N,          /* { */
	G|L,            /* | */
	J|M|P,          /* } */
	F|P|H,          /* ~ */
	0x0  /* DEL */
};




#endif