#include <stdint.h>
#include <stdio.h>
#include "evo_debug.h"
#include "usb/usb.h"

#include "datalink.h"
#include "command.h"
#include "print_funcs.h"


void print_frame_header(evoframe_header_t * header)
{
    print_dbg("\n--Header--\n");
    print_dbg("Marker: ");  print_dbg_char_hex(header->marker); print_dbg("\n");
    print_dbg("Length (bytes): ");  print_dbg_hex(header->len_bytes); print_dbg("\n");
    print_dbg("Sequence number: ");  print_dbg_char_hex(header->seq); print_dbg("\n");
    print_dbg("Pid : ");  print_dbg_char_hex(header->pid); print_dbg("\n");
    print_dbg("Type : ");  print_dbg_char_hex(header->type); print_dbg("\n\n");
}

void print_frame_payload(uint8_t * data, int len)
{
    int i;
    print_dbg("\n--Payload--\n");
    for(i = 0; i < len; i++) {
        print_dbg_char_hex(data[i]);
    }
    print_dbg("\n----\n");
}


int check_header(evoframe_header_t * header)
{
    return 0;
    if (header->marker != HEADER_MARKER) {
        //PRINT_DEBUG("parse_header: Bad marker\n");
        return BAD_MARKER;
    }

    if (header->len_bytes < 0) {
        //PRINT_DEBUG("parse_header: Bad length\n");
        return BAD_LENGTH;
    }

    return 0;
}

evoframe_t * recv_frame(void)
{   
    int err;
    /* malloc a frame struct */
    evoframe_t * frame = malloc(sizeof(evoframe_t));

    /* First receive header, and do some quick sanity checking */
    void * head = malloc(sizeof(evoframe_header_t));
    datalink_read(head, sizeof(evoframe_header_t));

    frame->header = (evoframe_header_t *)head; 

    if((err = check_header(frame->header)) < 0) {
        print_dbg("recv_frame: check_header failed\n");
        free(frame);
        free(head);
        return NULL;
    } 

    /* Header OK, malloc room for data and receive it */
    frame->data = malloc(frame->header->len_bytes);
    datalink_read(frame->data, frame->header->len_bytes);

    return frame;
}

void delete_frame(evoframe_t * frame)
{
    //First, free the header.
    free(frame->header);
    //Then, free data.
    free(frame->data);
    //Finally, free the frame itself. 
    free(frame);
}


int send_frame(evoframe_t * frame, int payload_size) 
{
    uint8_t * head = (uint8_t *)frame->header;
    datalink_write(head, sizeof(evoframe_header_t));

    //now, write data.
    datalink_write(frame->data, payload_size);

    return 0;
}


evoframe_header_t * create_frame_header(uint8_t marker, uint32_t length, uint8_t pid, uint8_t type)
{
    evoframe_header_t * head = malloc(sizeof(evoframe_header_t));
    head->marker = marker;
    head->len_bytes = (length);
    head->pid = pid;
    head->type = type;

    return head;
}

int datalink_read(uint8_t * bytes, int len)
{
    usb_read(bytes, len, 0); 
    return 0;
}

int datalink_write(uint8_t * bytes, int len) 
{
   usb_write(bytes, len, 0);
   return 0;
}
