#include <stdint.h>

#include "config.h"
#include "fpga.h"
#include "cycle_counter.h"
#include "fpga_config.h"
#include "board.h"
#include "print_funcs.h"

//#define DEBUGMODE 
void apply_vector(uint16_t * bytes, int wait, uint8_t * response)
{
    uint16_t * fpga = (uint16_t * )FPGA_ADDR;
    int bank;
    //Configure the banks by writing to the pin-config area of the FPGA
    for(bank = 0; bank < NBANKS; bank++) {
         fpga[PINCFG_OFFSET + bank] = bytes[bank]; 
    }
    //and writing the command register with the approproate value.
    fpga[CMD_OFFSET] = CMD_SEND_PATTERN;
   
    #ifdef DEBUGMODE
        print_dbg("apply_vector_2: config written to banks:\n");
        for(bank = 0; bank < NBANKS; bank++) {
             print_dbg_hex(fpga[PINCFG_OFFSET + bank]);
             print_dbg("\n");
        }
    #endif
    
    //Now, we hold a number of ms before returning. Experiments shows this
    //needs to be at least 3 ms for the arbitration to work. Not good...
    //uh, actually... this really sucks =)
    #ifdef DEBUGMODE
    print_dbg("apply_vector_2: waiting for ");
    print_dbg_hex(wait);
    print_dbg(" ms \n");
    #endif
    cpu_delay_ms(wait, FCPU);

    while(fpga[CMD_OFFSET] != CMD_NOOP) {
        #ifdef DEBUGMODE
        print_dbg("waiting for noop\n"); 
        #endif
    }
    
    //Next up, read back data.
    fpga[CMD_OFFSET] = CMD_READ_PINS;
  
     //Wait until command completes.
    while(fpga[CMD_OFFSET] != CMD_NOOP) {
        #ifdef DEBUGMODE
        print_dbg("waiting for read complete\n"); 
        #endif
    } 

    //Pin data ready, read it back to our memory. 
    //uint16_t * respvector = (uint16_t *)(&responses[n]);
    for(bank = 0; bank < NBANKS; bank++) {
        *(response + bank) = fpga[PINVAL_OFFSET + bank];
        #ifdef DEBUGMODE
        print_dbg_hex(*(response + bank));
        print_dbg("\n");
        #endif
    }
  
    #ifdef DEBUG
        int i;
        print_dbg("apply_vector_2: stored response:\n");
        uint16_t * lol = (uint16_t *)response;
        for(i = 0; i < NBANKS; i++) {
            print_dbg_hex(*(lol + i));
        }
        print_dbg("\n");
    #endif
}

