#include <avr32/io.h>
#include <stdint.h>
#include <string.h>

#include "intc.h"
#include "fpga_config.h"
//#include "fpga_config_data.h"
#include "gpio.h"
#include "tc.h"
#include "board.h"
#include "bit_twiddle.h"
#include "print_funcs.h"
#include "cycle_counter.h"
#include "datalink.h"

volatile uint32_t sent_bits = 0;
volatile uint8_t flank_toggle = 0;

volatile int configured;
uint8_t * configdata;
int config_bits = 0;


void program_fpga(evoframe_t * cmd)
{
    uint32_t bytes;
    uint32_t chunksize;
    memcpy(&bytes, cmd->data, 4);
    memcpy(&chunksize, cmd->data + 4, 4);
   
    print_dbg("Configuring fpga with ");
    print_dbg_hex(bytes);
    print_dbg(" bytes, in chunks of ");
    print_dbg_hex(chunksize);
    print_dbg("\n");

    int chunks = bytes/chunksize;
    int rem = bytes%chunksize;
   
    int chunk;
    evoframe_t * frame;
    uint8_t * configdata;    

    /*We'll receive frames of size chunksize here, and then clock this into the FPGA.
    This will most likely lead to our fucking failure, but hey.
    Configuration of the FPGA happens in these stages:
    0. Start clock
    1. We pulse PROG_B high
    2. INIT_B will now go high, indicating that FPGA is happy. 
    3. Send a clock flank and a bit.
    4. Check if DONE goes high or INIT_B goes low (meaning an error)
    */ 

    //Set mode pins
    gpio_set_gpio_pin(24);
    gpio_set_gpio_pin(29);
    gpio_set_gpio_pin(30);

    //clear the clock
    gpio_clr_gpio_pin(CCLK);
    gpio_clr_gpio_pin(PROG_B);

    while(gpio_get_pin_value(INIT_B) == 1);
    while(gpio_get_pin_value(DONE) == 1);

    gpio_set_gpio_pin(PROG_B);

    cpu_delay_ms(1, FOSC0);
        
    int curr_byte;
    int bytes_in_chunk = chunksize;
    for(chunk = 0; chunk <= chunks; chunk++) {
        if(chunk == chunks) {
            bytes_in_chunk = rem;
        } 

        frame = recv_frame();
        configdata = (uint8_t *)frame->data;

        if(gpio_get_pin_value(INIT_B) == 0) {
            print_dbg("Error in configuration. (try again?)\n");
        }

        for(curr_byte = 0; curr_byte < bytes_in_chunk; curr_byte++) {
            int bit;
            for(bit = 7; bit >= 0; bit--) {
                gpio_clr_gpio_pin(CCLK);
                if(GETBIT(configdata[curr_byte], bit) == 1) {
                    gpio_set_gpio_pin(SERIAL_OUT);
                } else {
                    gpio_clr_gpio_pin(SERIAL_OUT);
                }
                gpio_set_gpio_pin(CCLK);
            }
        }

        delete_frame(frame);
        
        if(gpio_get_pin_value(DONE) == 1) {
           break;
        }
    }

    //Clock 1's until we get something...
    print_dbg("pushed bits\n");
    gpio_set_gpio_pin(SERIAL_OUT);
    while(gpio_get_pin_value(DONE) == 0) {
        
        gpio_clr_gpio_pin(CCLK);
        gpio_set_gpio_pin(CCLK);
    }

    //Pull down the reset (It's on HSWAP)
    print_dbg("Resetting\n");
    gpio_clr_gpio_pin(HSWAP);
    cpu_delay_ms(2, FOSC0);
    gpio_set_gpio_pin(HSWAP);

 
    print_dbg("Configuration succeeded.\n");
    
}

