#ifndef _FPGA_SMC_H_
#define _FPGA_SMC_H_

//! SMC Peripheral Memory Size in Bytes
#define EXT_SM_SIZE            0x80000 //19 address bits.

//! SMC Data Bus Width
#define SMC_DBW                16

//! Whether 8-bit SM chips are connected on the SMC
#define SMC_8_BIT_CHIPS        FALSE

// NCS setup time. Unit: ns.
#define NCS_WR_SETUP            256 //no need to wait with address

// NCS pulse time. Unit: ns.
#define NCS_WR_PULSE            480 //freezes anyway, so we'll just hold one fpga cycle for good measure


// NCS hold time. Unit: ns.
#define NCS_WR_HOLD             256 //no need to hold

// NWE setup time. Unit: ns.
#define NWE_SETUP               256 //no need to wait with putting address on bus after write-enable

// NWE pulse time. Unit: ns.
#define NWE_PULSE               480 //hold for one cycle at least. 

// NWE hold time. Unit: ns.
#define NWE_HOLD                128 //release immediatly.

// Write cycle time. Unit: ns.
#define NWE_CYCLE               Max((NCS_WR_SETUP + NCS_WR_PULSE + NCS_WR_HOLD),(NWE_SETUP + NWE_PULSE + NWE_HOLD))

// NCS setup time. Unit: ns.
#define NCS_RD_SETUP            256 //no need to wait with putting address on bus after cs, because we sample cs first, then address. 

// NCS pulse time. Unit: ns.
#define NCS_RD_PULSE            480 //hold pulse, we freeze anyway

// NCS hold time. Unit: ns.
#define NCS_RD_HOLD             256 //remove immediatly

// NRD setup time. Unit: ns.
#define NRD_SETUP               256 //push address onto bus alongside cs and re

// NRD pulse time. Unit: ns.
#define NRD_PULSE               480 //hold pulse for one fpga cycle at least. will freeze anyway.

// NRD hold time. Unit: ns.
#define NRD_HOLD                256 //release immediatly when address is gone. 

// Read cycle time. Unit: ns.
#define NRD_CYCLE               Max((NCS_RD_SETUP + NCS_RD_PULSE + NCS_RD_HOLD),(NRD_SETUP + NRD_PULSE + NRD_HOLD))

// Data float time
#define TDF_CYCLES              0
#define TDF_OPTIM               DISABLED

// Page mode
#define PAGE_MODE               DISABLED
#define PAGE_SIZE               0

//! Whether read is controlled by NCS or by NRD
#define NCS_CONTROLLED_READ     FALSE

//! Whether write is controlled by NCS or by NWE
#define NCS_CONTROLLED_WRITE    FALSE

//! Whether to use the NWAIT pin
//#define NWAIT_MODE_GLOBAL
#define NWAIT_MODE              AVR32_SMC_EXNW_MODE_FROZEN

#endif 
