#include "usb/usb.h"
#include "setup_hw.h"
#include "command.h"
#include "print_funcs.h"
//#include "evo_debug.h"
#include "gpio.h"
#include "usart.h"
#include "pm.h"
#include "tc.h"
#include "usb_task.h"
#include "fpga_config.h"
#include "smc.h"
#include <stdint.h>
#include "cycle_counter.h"
#include "intc.h"

#define USB_DEVICE_FEATURE  ENABLED
int running = 1;


/*  Global buffers used for storing configurations on the card. 
    We receive the configuration bits from the host computer */ 
uint32_t num_vectors;
uint8_t ** configs;
uint8_t ** responses;

volatile uint16_t tc_tick = 0;
volatile uint16_t printt = 0;


void setup_uc3(void) 
{
    gpio_set_gpio_pin(PROG_B);
    gpio_clr_gpio_pin(HSWAP);
    setup_clocks();
    setup_debug_usart(); //debug usart
    
    setup_intc(); //interrupt controller
    setup_usb(); //turn on usb
    
    smc_init(FCPU); //turn on memory controller

    setup_tc(); //Start timer/counter

}


int main(void) {
    setup_uc3();
    evoframe_t * current_cmd;
    
    //fpga out of reset
   gpio_set_gpio_pin(HSWAP);
  
   print_dbg("Entering loop\n");

   while(running) { 
        current_cmd = recv_frame();
        print_dbg("Got command\n");
        execute_command(current_cmd);
        delete_frame(current_cmd);
    }
    return 0;
}
