#ifndef __PERIPHERALS_H__
#define __PERIPHERALS_H__

/* Settings and defines for the timer/counter-module */
#define TC_IRQ_GROUP        AVR32_TC_IRQ_GROUP
#define TC_IRQ              AVR32_TC_IRQ0
#define TC_CHANNEL 0
#define FPBA FOSC0

volatile avr32_tc_t *tc = &(AVR32_TC);

// Options for waveform genration.
static const tc_waveform_opt_t WAVEFORM_OPT =
{
    .channel  = TC_CHANNEL,              // Channel selection.
    .bswtrg   = TC_EVT_EFFECT_NOOP,      // Software trigger effect on TIOB.
    .beevt    = TC_EVT_EFFECT_NOOP,      // External event effect on TIOB.
    .bcpc     = TC_EVT_EFFECT_NOOP,     // RC compare effect on TIOB.
    .bcpb     = TC_EVT_EFFECT_NOOP,    // RB compare effect on TIOB.
    .aswtrg   = TC_EVT_EFFECT_NOOP,    // Software trigger effect on TIOA.
    .aeevt    = TC_EVT_EFFECT_NOOP,    // External event effect on TIOA.
    .acpc     = TC_EVT_EFFECT_NOOP,    // RC compare effect on TIOA: toggle.
    .acpa     = TC_EVT_EFFECT_NOOP,    // RA compare effect on TIOA: toggle (other possibilities are none, set and clear)
    .wavsel   = TC_WAVEFORM_SEL_UP_MODE_RC_TRIGGER,// Waveform selection: Up mode with automatic trigger(reset) on RC compare.
    .enetrg   = FALSE,                  // External event trigger enable.
    .eevt     = 0,                      // External event selection.
    .eevtedg  = TC_SEL_NO_EDGE,         // External event edge selection.
    .cpcdis   = FALSE,                  // Counter disable when RC compare.
    .cpcstop  = FALSE,              // Counter clock stopped with RC compare.
    .burst    = FALSE,                  // Burst signal selection.
    .clki     = FALSE,                  // Clock inversion.
    .tcclks   = TC_CLOCK_SOURCE_TC3     // Internal source clock 3, connected to fPBA / 8.
};


#endif
