#include <avr32/io.h>
#include "board.h" //includes the evoboard.h file
#include "compiler.h" //lots of common stuff for AVR chips, Set_system_register, etc...
#include "debug.h"

/* Driver includes */
#include "gpio.h"
#include "intc.h" 
#include "pm.h"
#include "tc.h"
#include "flashc.h"
#include "print_funcs.h"

/* uC-program includes */
#include "command.h"
#include "usb/usb.h"
#include "setup_hw.h"
#include "nlao_cpu.h" //newlib requirement
#include "cycle_counter.h"



/* Settings and defines for the timer/counter-module */
#define TC_IRQ_GROUP        AVR32_TC_IRQ_GROUP
#define TC_IRQ              AVR32_TC_IRQ0
#define TC_CHANNEL 0
#define FPBA FOSC0


void setup_tc()
{
    volatile avr32_tc_t *tc = &(AVR32_TC);
    static const tc_interrupt_t TC_INTERRUPT =
    {
        .etrgs = 0,
        .ldrbs = 0,
        .ldras = 0,
        .cpcs  = 1,
        .cpbs  = 0,
        .cpas  = 0,
        .lovrs = 0,
        .covfs = 0
    };

    // Options for waveform genration, taken from framework code. 
    static const tc_waveform_opt_t WAVEFORM_OPT =
    {
        .channel  = TC_CHANNEL,              // Channel selection.
        .bswtrg   = TC_EVT_EFFECT_NOOP,      // Software trigger effect on TIOB.
        .beevt    = TC_EVT_EFFECT_NOOP,      // External event effect on TIOB.
        .bcpc     = TC_EVT_EFFECT_NOOP,     // RC compare effect on TIOB.
        .bcpb     = TC_EVT_EFFECT_NOOP,    // RB compare effect on TIOB.
        .aswtrg   = TC_EVT_EFFECT_NOOP,    // Software trigger effect on TIOA.
        .aeevt    = TC_EVT_EFFECT_NOOP,    // External event effect on TIOA.
        .acpc     = TC_EVT_EFFECT_NOOP,    // RC compare effect on TIOA: toggle.
        .acpa     = TC_EVT_EFFECT_NOOP,    // RA compare effect on TIOA: toggle (other possibilities are none, set and clear)
        .wavsel   = TC_WAVEFORM_SEL_UP_MODE_RC_TRIGGER,// Waveform selection: Up mode with automatic trigger(reset) on RC compare.
        .enetrg   = FALSE,                  // External event trigger enable.
        .eevt     = 0,                      // External event selection.
        .eevtedg  = TC_SEL_NO_EDGE,         // External event edge selection.
        .cpcdis   = FALSE,                  // Counter disable when RC compare.
        .cpcstop  = FALSE,              // Counter clock stopped with RC compare.
        .burst    = FALSE,                  // Burst signal selection.
        .clki     = FALSE,                  // Clock inversion.
        .tcclks   = TC_CLOCK_SOURCE_TC3     // Internal source clock 3, connected to fPBA / 8.
    };

    Disable_global_interrupt();
    INTC_register_interrupt(&stability_read, TC_IRQ, AVR32_INTC_INT0);
    Enable_global_interrupt();

    tc_init_waveform(tc, &WAVEFORM_OPT);         // Initialize the timer/counter waveform.

    tc_configure_interrupts(tc, TC_CHANNEL, &TC_INTERRUPT);
    tc_write_rc(tc, TC_CHANNEL, 2000); // Set RC value, 1 tick per ms.
    tc_start(tc, TC_CHANNEL);
}


void setup_intc()
{
    /* Exception vector base address */
    Set_system_register(AVR32_EVBA, (int)&_evba);
    Enable_global_exception();
    INTC_init_interrupts();
}

void setup_clocks()
{
    volatile avr32_pm_t * pm = &AVR32_PM;
    pm_switch_to_osc0(pm, FOSC0, AVR32_PM_OSCCTRL0_STARTUP_128_RCOSC);

    /* Set up system frequencies */
    pm_freq_param_t freq;
    freq.cpu_f = FCPU;
    freq.pba_f = FCPU/4;
    freq.osc0_f = FOSC0;
    freq.osc0_startup = OSC0_STARTUP;
    pm_configure_clocks(&freq);

    set_cpu_hz(FCPU);
}

void setup_debug_usart()
{
    init_dbg_rs232(FCPU);
    print_dbg("Debug USART alive!\n");
}

void setup_usb()
{
    init_usb();
}
