#include "usb.h"
#include <stdint.h>
#include "usb_drv.h"
#include "evo_debug.h"
#include "conf_usb.h"
#include "usb_descriptors.h"
#include "usb_standard_request.h"
#include "usb_task.h"  //includes init-routines for USB
#include "pm.h"
#include "print_funcs.h"


uint8_t rx_bytes;
uint8_t tx_bytes;
static volatile uint16_t sof_cnt;
uint8_t usb_rx_new;
uint8_t usb_tx_new;


int usb_enumerated(void)
{
    return Is_device_enumerated();
}

void usb_sof_action(void)
{
    //do naighting
    sof_cnt++;
}
void init_usb()
{

    //Hardware
    pm_configure_usb_clock(); 
    usb_task_init();
    
    //Turn on start-of-frame-detection interrupt
    if(Is_usb_device()) {
        Usb_enable_sof_interrupt(); 
        sof_cnt = 0;
        //PRINT_DEBUG("Initialized start-of-frame interrupt\n");
    }

    rx_bytes = 0;
    tx_bytes = 0;

    usb_rx_new = 1;
    usb_tx_new = 1;
}

void usb_read(void * buf, int bytes, int timeout) 
{
    int bytes_read = 0;
    uint8_t * buffer = buf;

    while(bytes_read < bytes) {
        usb_task();
        if(!Is_device_enumerated()) {
            continue;
            //PRINT_DEBUG("Attempted to read from unemuerated device\n");
        }

        if(sof_cnt >= 50) {
            sof_cnt = 0;
            usb_flush();
        }

        /* Treat the buffer as a byte-sequence */ 
        /* Read byte-for-byte into buffer */
        if(usb_data_ready()) {
            buffer[bytes_read] = usb_rx_byte();
           bytes_read++;
       }
    }
}

void cdc_task(void)
{
    uint8_t c; 
        if(!Is_device_enumerated()) { return; }

        if(sof_cnt >= 50) {
            sof_cnt = 0;
            usb_flush();
        }

        if(usb_data_ready()) {
            c = usb_rx_byte();
            
        }
}

uint8_t usb_rx_byte(void)
{
    register uint8_t data_rx;
    while (!usb_data_ready()){ 
  //      print_dbg("usb_rx_byte waiting\n");
    } 
    
    data_rx = Usb_read_endpoint_data(RX_EP, 8);

    if (Usb_byte_count(RX_EP) == 0) {
        Usb_ack_out_received_free(RX_EP);
        usb_rx_new = 1;
    } 
    return data_rx;
}


void usb_write(void * buf, int bytes, int timeout) 
{
   uint8_t * buffer = (uint8_t * )buf;
   int bytes_written = 0;
    while(bytes_written < bytes) {
        usb_task();
        if(!Is_device_enumerated()) {
            continue;
        }
        usb_tx_byte(buffer[bytes_written]);
        bytes_written++;
   }
}

int usb_tx_ready(void)
{
    if(!Is_usb_write_enabled(TX_EP)) {
        return 0;
    } else {
        return 1;
    }
}

void usb_tx_byte(uint8_t byte) 
{
    while(!usb_tx_ready());
    if(tx_bytes == 0) {
        Usb_reset_endpoint_fifo_access(TX_EP);
    }
    Usb_write_endpoint_data(TX_EP, 8, byte);
    tx_bytes++;
    
    /* Endpoint is full, flush fifo */
    if(!usb_tx_ready()) { 
        usb_flush();
    }
}

void usb_flush(void) 
{
    if (Usb_nb_busy_bank(TX_EP) != 0) {
        return; //old flush
    }
    Usb_ack_in_ready_send(TX_EP);
    usb_tx_new = 1;
}

int usb_data_ready(void)
{
    /* Check if there is data still in the FIFO */
    if (Is_usb_out_received(RX_EP)) {
        if (Usb_byte_count(RX_EP) != 0) {
            if (usb_rx_new) {
                Usb_reset_endpoint_fifo_access(RX_EP);
                usb_rx_new = 0;
            }
            return 1;
        }
        Usb_ack_out_received_free(RX_EP);
        usb_rx_new = 1;
    }
    return 0;
}

//void usb_tx_data(){};
//void usb_rx_data(){char out};
