# Hey Emacs, this is a -*- makefile -*-

# The purpose of this file is to define the build configuration variables used
# by the generic Makefile. See Makefile header for further information.

# Copyright (c) 2009 Atmel Corporation. All rights reserved. 
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright notice, 
# this list of conditions and the following disclaimer in the documentation 
# and/or other materials provided with the distribution.
#
# 3. The name of Atmel may not be used to endorse or promote products derived 
# from this software without specific prior written permission.  
#
# 4. This software may only be redistributed and used in connection with an Atmel 
# AVR product. 
#
# THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED 
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE 
# EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR 
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
#

# Base paths
PRJ_PATH = ../../../..
APPS_PATH = $(PRJ_PATH)/APPLICATIONS
BRDS_PATH = $(PRJ_PATH)/BOARDS
COMP_PATH = $(PRJ_PATH)/COMPONENTS
DRVR_PATH = $(PRJ_PATH)/DRIVERS
SERV_PATH = $(PRJ_PATH)/SERVICES
UTIL_PATH = $(PRJ_PATH)/UTILS

# Demo paths
FREERTOS_PATH = $(SERV_PATH)/FREERTOS
FREERTOS_PORT_PATH = $(FREERTOS_PATH)/Source/portable/GCC/AVR32_UC3
FREERTOS_MEM_PATH = $(FREERTOS_PATH)/Source/portable/MemMang
DEMO_PATH = $(APPS_PATH)/EVK1100-CONTROL-PANEL
UTILS_PATH = $(DEMO_PATH)/UTILS
LOCAL_PATH = $(DEMO_PATH)/LOCALCTRL
SENSOR_PATH = $(DEMO_PATH)/SENSORS
CFG_PATH = $(DEMO_PATH)/CONFIG
MMI_PATH = $(DEMO_PATH)/MMI
ETH_PATH = $(DEMO_PATH)/NETWORK
WEB_PATH = $(ETH_PATH)/WEBSERVER
TFTP_PATH = $(ETH_PATH)/TFTPSERVER
SMTP_PATH = $(ETH_PATH)/SMTPCLIENT
POP3_PATH = $(ETH_PATH)/POP3CLIENT
LWIP_PATH = $(SERV_PATH)/LWIP/lwip-1.3.2/src
LWIP_PORT_PATH = $(SERV_PATH)/LWIP/lwip-port-1.3.2/AT32UC3A
USB_PATH = $(DEMO_PATH)/USB
USB_DRVR_PATH = $(DRVR_PATH)/USBB
USB_ENUM_PATH = $(USB_DRVR_PATH)/ENUM
USB_ENUMD_PATH = $(USB_ENUM_PATH)/DEVICE
USB_ENUMH_PATH = $(USB_ENUM_PATH)/HOST
USB_CLASS_PATH = $(SERV_PATH)/USB/CLASS
USB_CLASS_MS_PATH = $(USB_CLASS_PATH)/MASS_STORAGE
USB_APPLI = $(USB_PATH)/APPLI
USB_APPLI_ENUM = $(USB_APPLI)/ENUM
USB_APPLI_ENUM_DEVICE = $(USB_APPLI_ENUM)/DEVICE
USB_APPLI_MS = $(USB_APPLI)/MASS_STORAGE
EVK1100_PATH = $(BRDS_PATH)/EVK1100

# CPU architecture: {ap|ucr1|ucr2}
ARCH = ucr2

# Part: {none|ap7xxx|uc3xxxxx}
PART = uc3a0512

# Flash memories: [{cfi|internal}@address,size]...
FLASH = internal@0x80000000,512Kb

# Clock source to use when programming: [{xtal|extclk|int}]
PROG_CLOCK = xtal

# Device/Platform/Board include path
PLATFORM_INC_PATH = \
  $(BRDS_PATH)/

# Target name: {*.a|*.elf}
TARGET = $(PART)-ctrlpanel.elf

# Definitions: [-D name[=definition]...] [-U name...]
# Things that might be added to DEFS:
#   BOARD                            Board used: see $(BRDS_PATH)/board.h
#   EXT_BOARD                        Extension board used (if any): see $(BRDS_PATH)/board.h
#   FREERTOS_USED                    Whether FreeRTOS is used or not
#   NW_INTEGRATED_IN_CONTROL_PANEL   Enable/Disable the Control Panel TCP/IP feature
#   HTTP_USED                        Enable/Disable the WEB server
#   TFTP_USED                        Enable/Disable the TFTP server
#   SMTP_USED                        Enable/Disable the SMTP client
#   USB_ENABLE                       Enable/Disable the Control Panel USB feature
#   MMILCD_ENABLE                    Enable/Disable the MMI+LCD on board feature
#   DISPLAY_MMI_SECOND               Enable/Disable the display of second on the MMI
#   -D TCPIP_DBG_MODE                Enable the usage of the leds to help some TCP/IP debug.
DEFS = -D BOARD=EVK1100 \
       -D FREERTOS_USED \
       -D HTTP_USED=1 \
       -D TFTP_USED=0 \
       -D SMTP_USED=0 \
       -D NW_INTEGRATED_IN_CONTROL_PANEL \
       -D USB_ENABLE \
       -D DISPLAY_MMI_SECOND=1\
       -D MMILCD_ENABLE

# Include path
INC_PATH = \
  $(UTIL_PATH)/ \
  $(UTIL_PATH)/PREPROCESSOR/ \
  $(UTIL_PATH)/LIBS/NEWLIB_ADDONS/INCLUDE/ \
  $(SERV_PATH)/USB/ \
  $(SERV_PATH)/USB/CLASS/DFU/EXAMPLES/ISP/BOOT/ \
  $(DRVR_PATH)/INTC/ \
  $(DRVR_PATH)/TC/ \
  $(DRVR_PATH)/PM/ \
  $(DRVR_PATH)/GPIO/ \
  $(DRVR_PATH)/FLASHC/ \
  $(DRVR_PATH)/ADC/ \
  $(DRVR_PATH)/SPI/ \
  $(DRVR_PATH)/MACB/ \
  $(DRVR_PATH)/PWM/ \
  $(DRVR_PATH)/USART/ \
  $(COMP_PATH)/MEMORY/DATA_FLASH/AT45DBX/ \
  $(COMP_PATH)/MEMORY/SD_MMC/SD_MMC_SPI/ \
  $(SERV_PATH)/DELAY/ \
  $(SERV_PATH)/MEMORY/CTRL_ACCESS/ \
  $(SERV_PATH)/FAT/ \
  $(DEMO_PATH)/ \
  $(DEMO_PATH)/SERIAL/ \
  $(DEMO_PATH)/ACTUATORS/ \
  $(SENSOR_PATH)/ \
  $(DEMO_PATH)/SHELL/ \
  $(UTILS_PATH)/ \
  $(LOCAL_PATH)/ \
  $(FREERTOS_PATH)/Source/include/ \
  $(FREERTOS_PORT_PATH)/ \
  $(FREERTOS_MEM_PATH)/ \
  $(MMI_PATH)/ \
  $(COMP_PATH)/DISPLAY/DIP204/ \
  $(ETH_PATH)/ \
  $(CFG_PATH)/ \
  $(LWIP_PATH)/include/ \
  $(LWIP_PATH)/include/ipv4/ \
  $(LWIP_PORT_PATH)/include/ \
  $(WEB_PATH)/ \
  $(TFTP_PATH)/ \
  $(SMTP_PATH)/ \
  $(USB_DRVR_PATH)/ \
  $(USB_ENUM_PATH)/ \
  $(USB_ENUMD_PATH)/ \
  $(USB_ENUMH_PATH)/ \
  $(USB_CLASS_MS_PATH)/SCSI_DECODER/ \
  $(USB_CLASS_MS_PATH)/HOST_MEM/ \
  $(USB_APPLI)/ \
  $(USB_APPLI_ENUM)/ \
  $(USB_APPLI_ENUM_DEVICE)/ \
  $(USB_APPLI_MS)/

# C source files

LWIP_SRC = \
  $(LWIP_PATH)/core/dns.c \
  $(LWIP_PATH)/core/init.c \
  $(LWIP_PATH)/core/mem.c \
  $(LWIP_PATH)/core/memp.c \
  $(LWIP_PATH)/core/netif.c \
  $(LWIP_PATH)/core/pbuf.c \
  $(LWIP_PATH)/core/raw.c \
  $(LWIP_PATH)/core/stats.c \
  $(LWIP_PATH)/core/sys.c \
  $(LWIP_PATH)/core/tcp.c \
  $(LWIP_PATH)/core/tcp_in.c \
  $(LWIP_PATH)/core/tcp_out.c \
  $(LWIP_PATH)/core/ipv4/autoip.c \
  $(LWIP_PATH)/core/ipv4/igmp.c \
  $(LWIP_PATH)/core/ipv4/inet.c \
  $(LWIP_PATH)/core/ipv4/inet_chksum.c \
  $(LWIP_PATH)/core/ipv4/ip.c \
  $(LWIP_PATH)/core/ipv4/ip_addr.c \
  $(LWIP_PATH)/core/ipv4/icmp.c \
  $(LWIP_PATH)/api/netbuf.c \
  $(LWIP_PATH)/api/netdb.c \
  $(LWIP_PATH)/api/netifapi.c \
  $(LWIP_PATH)/api/sockets.c \
  $(LWIP_PATH)/api/tcpip.c \
  $(LWIP_PATH)/api/api_msg.c \
  $(LWIP_PATH)/api/err.c \
  $(LWIP_PATH)/api/api_lib.c \
  $(LWIP_PATH)/netif/etharp.c \
  $(LWIP_PATH)/core/udp.c \
  $(LWIP_PATH)/core/ipv4/ip_frag.c

FAT_SRC = \
  $(DRVR_PATH)/SPI/spi.c \
  $(COMP_PATH)/MEMORY/DATA_FLASH/AT45DBX/at45dbx.c \
  $(COMP_PATH)/MEMORY/DATA_FLASH/AT45DBX/at45dbx_mem.c \
  $(COMP_PATH)/MEMORY/SD_MMC/SD_MMC_SPI/sd_mmc_spi.c \
  $(COMP_PATH)/MEMORY/SD_MMC/SD_MMC_SPI/sd_mmc_spi_mem.c \
  $(SERV_PATH)/MEMORY/CTRL_ACCESS/ctrl_access.c \
  $(SERV_PATH)/FAT/fat.c \
  $(SERV_PATH)/FAT/fat_unusual.c \
  $(SERV_PATH)/FAT/file.c \
  $(SERV_PATH)/FAT/navigation.c \
  $(SERV_PATH)/FAT/fsaccess.c

USB_SRC = \
  $(USB_DRVR_PATH)/usb_drv.c \
  $(USB_ENUM_PATH)/usb_task.c \
  $(USB_ENUMD_PATH)/usb_device_task.c \
  $(USB_ENUMD_PATH)/usb_standard_request.c \
  $(USB_ENUMH_PATH)/usb_host_enum.c \
  $(USB_ENUMH_PATH)/usb_host_task.c \
  $(USB_CLASS_MS_PATH)/SCSI_DECODER/scsi_decoder.c \
  $(USB_CLASS_MS_PATH)/HOST_MEM/host_mem.c \
  $(USB_APPLI_ENUM)/DEVICE/usb_descriptors.c \
  $(USB_APPLI_ENUM)/DEVICE/usb_specific_request.c \
  $(USB_APPLI_MS)/device_mass_storage_task.c \
  $(USB_APPLI_MS)/host_mass_storage_task.c \
  $(USB_APPLI)/usbsys.c

CSRCS = \
  $(DRVR_PATH)/INTC/intc.c \
  $(DRVR_PATH)/TC/tc.c \
  $(DRVR_PATH)/PM/pm.c \
  $(DRVR_PATH)/GPIO/gpio.c \
  $(DRVR_PATH)/FLASHC/flashc.c \
  $(DRVR_PATH)/ADC/adc.c \
  $(DRVR_PATH)/MACB/macb.c \
  $(DRVR_PATH)/PWM/pwm.c \
  $(DRVR_PATH)/USART/usart.c \
  $(DEMO_PATH)/main.c \
  $(DEMO_PATH)/SERIAL/serial.c \
  $(UTILS_PATH)/printf-stdarg.c \
  $(FREERTOS_PATH)/Source/tasks.c \
  $(FREERTOS_PATH)/Source/queue.c \
  $(FREERTOS_PATH)/Source/list.c \
  $(FREERTOS_PATH)/Source/croutine.c \
  $(FREERTOS_PORT_PATH)/port.c \
  $(FREERTOS_MEM_PATH)/heap_3.c \
  $(LWIP_SRC) \
  $(LWIP_PORT_PATH)/sys_arch.c \
  $(LWIP_PORT_PATH)/netif/ethernetif.c \
  $(CFG_PATH)/config_file.c \
  $(WEB_PATH)/BasicWEB.c \
  $(TFTP_PATH)/BasicTFTP.c \
  $(SMTP_PATH)/BasicSMTP.c \
  $(ETH_PATH)/ethernet.c \
  $(SERV_PATH)/DELAY/delay.c \
  $(FAT_SRC) \
  $(UTILS_PATH)/tracedump.c \
  $(LOCAL_PATH)/supervisor.c \
  $(DEMO_PATH)/SHELL/com1shell.c \
  $(DEMO_PATH)/SHELL/shell.c \
  $(DEMO_PATH)/SHELL/fscmds.c \
  $(SENSOR_PATH)/sensor.c \
  $(DEMO_PATH)/ACTUATORS/actuator.c \
  $(LOCAL_PATH)/syscmds.c \
  $(SENSOR_PATH)/temperature.c \
  $(SENSOR_PATH)/potentiometer.c \
  $(SENSOR_PATH)/light.c \
  $(SENSOR_PATH)/pushb.c \
  $(SENSOR_PATH)/joystick.c \
  $(LOCAL_PATH)/cptime.c \
  $(LOCAL_PATH)/datalog.c \
  $(EVK1100_PATH)/led.c \
  $(DEMO_PATH)/ACTUATORS/cpled.c \
  $(DEMO_PATH)/ACTUATORS/lcd.c \
  $(USB_SRC) \
  $(MMI_PATH)/MMI.c \
  $(COMP_PATH)/DISPLAY/DIP204/dip204.c

# Assembler source files
ASSRCS = \
  $(SERV_PATH)/USB/CLASS/DFU/EXAMPLES/ISP/BOOT/trampoline.S \
  $(FREERTOS_PORT_PATH)/exception.S

# Library path
LIB_PATH = $(UTIL_PATH)/LIBS/NEWLIB_ADDONS/AT32UCR2/

# Libraries to link with the project
LIBS = newlib_addons-at32ucr2-speed_opt

# Linker script file if any
LINKER_SCRIPT = $(UTIL_PATH)/LINKER_SCRIPTS/AT32UC3A/0512/GCC/link_uc3a0512.lds

# Options to request or suppress warnings: [-fsyntax-only] [-pedantic[-errors]] [-w] [-Wwarning...]
# For further details, refer to the chapter "GCC Command Options" of the GCC manual.
WARNINGS = -Wall
# WARNINGS += -Wmissing-prototypes -Wstrict-prototypes -Wmissing-declarations \
#   -Wredundant-decls -Wnested-externs -Wundef

# Options for debugging: [-g]...
# For further details, refer to the chapter "GCC Command Options" of the GCC manual.
DEBUG = -g

# Options that control optimization: [-O[0|1|2|3|s]]...
# For further details, refer to the chapter "GCC Command Options" of the GCC manual.
OPTIMIZATION = -O3 -ffunction-sections -fdata-sections -fno-strict-aliasing

# Extra flags to use when preprocessing
CPP_EXTRA_FLAGS =

# Extra flags to use when compiling
C_EXTRA_FLAGS =

# Extra flags to use when assembling
AS_EXTRA_FLAGS =

# Extra flags to use when linking
LD_EXTRA_FLAGS = -Wl,--gc-sections -Wl,-e,_trampoline

# Documentation path
DOC_PATH = ../../DOC/

# Documentation configuration file
DOC_CFG = ../../DOCSRC/doxyfile.doxygen
