/* This source file is part of the ATMEL AVR-UC3-SoftwareFramework-1.7.0 Release */

/*This file is prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief generic 32-bit complex addition function
 *
 * - Compiler:           IAR EWAVR32 and GNU GCC for AVR32
 * - Supported devices:  All AVR32 devices.
 * - AppNote:
 *
 * \author               Atmel Corporation: http://www.atmel.com \n
 *                       Support and FAQ: http://support.atmel.no/
 *
 ******************************************************************************/

/* Copyright (c) 2009 Atmel Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an Atmel
 * AVR product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 *
 */

#include "dsp.h"
#include "preprocessor.h"

#if defined(FORCE_ALL_GENERICS) || \
    defined(FORCE_GENERIC_VECT32_COMPLEXADD) || \
    !defined(TARGET_SPECIFIC_VECT32_COMPLEXADD)

#define DSP32_COMPLEXADDITION(x_num, data) \
  vect1[x_num].real = vect2[x_num].real + vect3[x_num].real; \
  vect1[x_num].imag = vect2[x_num].imag + vect3[x_num].imag;

/*********************************************************************************************
 * This function is the ending function of the addition. It is used to add the last items of a vector.
 *********************************************************************************************/
#define DSP32_COMPLEXADD_KERNEL_X_FCT(x_num, data)    \
static void TPASTE2(dsp32_vect_complexadd_kernel_x, x_num)(dsp32_complex_t *vect1, dsp32_complex_t *vect2, dsp32_complex_t *vect3) \
{ \
  MREPEAT(x_num, DSP32_COMPLEXADDITION, ""); \
}
/*********************************************************************************************/

DSP32_COMPLEXADD_KERNEL_X_FCT(0, "")
DSP32_COMPLEXADD_KERNEL_X_FCT(1, "")
DSP32_COMPLEXADD_KERNEL_X_FCT(2, "")
DSP32_COMPLEXADD_KERNEL_X_FCT(3, "")

void dsp32_vect_complex_add(dsp32_complex_t *vect1, dsp32_complex_t *vect2, dsp32_complex_t *vect3, int size)
{
  typedef void (*complexadd_kernel_opti_t)(dsp32_complex_t *, dsp32_complex_t *, dsp32_complex_t *);
  static const complexadd_kernel_opti_t complexadd_end_kernel_opti[4] = {
    dsp32_vect_complexadd_kernel_x0,
    dsp32_vect_complexadd_kernel_x1,
    dsp32_vect_complexadd_kernel_x2,
    dsp32_vect_complexadd_kernel_x3
  };
  int n;

  for(n=0; n<size-3; n+=4)
  {
    DSP32_COMPLEXADDITION(n, "");
    DSP32_COMPLEXADDITION(n+1, "");
    DSP32_COMPLEXADDITION(n+2, "");
    DSP32_COMPLEXADDITION(n+3, "");
  }

  // Jump on different functions depending on the length of the vectors to compute
  complexadd_end_kernel_opti[size&0x3](&vect1[n], &vect2[n], &vect3[n]);
}

#endif
